#!BPY
"""
Name: 'Indigo 0.7-Final Export 1.21'
Blender: 243
Group: 'Export'
Tooltip: 'Export to Indigo 0.7-Final Scene Format (.igs)'
"""

# ***** BEGIN GPL LICENSE BLOCK *****
#
# -----------------------------------------------------------------
# Indigo 0.7-Final Export 1.21
# -----------------------------------------------------------------
#
# Authors:
# * Indigo exporter - Nick Chapman, Zuegs, Ewout Fernhout, Leope, Wedge
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENSE BLOCK *****

import Blender
from Blender import Draw, BGL, Scene, Window, Camera, Mesh, Material
from Blender.sys import makename, basename

def exportMesh(mesh,switch):
	global vlist, flist
	def addVertex(bvindex, coord, normal, uv):
		index = -1
		if (bvindex < len(vdata)):
			for ivindex in vdata[bvindex]:
				v = vlist[ivindex]
				if (abs(v[0][0]-coord[0])<0.0001) and (abs(v[0][1]-coord[1])<0.0001) and (abs(v[0][2]-coord[2])<0.0001) and (abs(v[1][0]-normal[0])<0.0001) and (abs(v[1][1]-normal[1])<0.0001) and (abs(v[1][2]-normal[2])<0.0001):
					if ((v[2]==[]) and (uv==[])) or ((abs(v[2][0]-uv[0])<0.0001) and (abs(v[2][1]-uv[1])<0.0001)): index = ivindex
		if index < 0:
			index = len(vlist)
			vlist.append([coord, normal, uv])
			while bvindex >= len(vdata): vdata.append([])
			vdata[bvindex].append(index)
		return index
	def getVertices(vlist):
		def getUVStr(uv):
			if uv != []: return "uv0=\"%.4f %.4f\"" %(uv[0], uv[1])
			else: return ""
		return "".join(map(lambda v: "\t\t\t<vertex pos=\"%.4f %.4f %.4f\"normal=\"%.4f %.4f %.4f\"%s/>\n" %(v[0][0], v[0][1], v[0][2], v[1][0], v[1][1], v[1][2], getUVStr(v[2])), vlist))
	def addFace(mindex, index0, index1, index2):
		while mindex >= len(flist): flist.append([])
		flist[mindex].append([index0, index1, index2])
	def getFaces():
		def getMaterialName(mindex):
			if mindex < len(mesh.materials): return mesh.materials[mindex].name
			else:
				print "Warning: faces without material"
				return "Default"
		return "".join(map(lambda fli: "\t\t\t<triangle_set>\n\t\t\t\t<material_name>%s</material_name>\n"%(getMaterialName(fli)) + "".join(map(lambda f: "\t\t\t\t<tri>%d %d %d</tri>\n"%(f[0], f[1], f[2]), flist[fli])) + "\t\t\t</triangle_set>\n" , xrange(len(flist))))
	if switch is 0:
		vdata = [] #[ii0, ii1, ii2, ...] lists indexed by Blender-Vertex-index
		vlist = [] #[coord, normal, uv] lists indexed by Indigo-Vertex-index
		flist = []
		for face in mesh.faces:
			iis = [-1, -1, -1, -1]
			for vi in xrange(len(face.v)):
				vert = face.v[vi]
				if face.smooth: normal = vert.no
				else: normal = face.no
				uv = []
				if mesh.faceUV:
					if len(face.uv) == len(face.v): uv = face.uv[vi]
				iis[vi] = addVertex(vert.index, vert.co, normal, uv)
			addFace(face.mat, iis[0], iis[1], iis[2])
			if len(face.v) == 4: addFace(face.mat, iis[2], iis[3], iis[0])
		str = "\t<mesh>\n"
		str += "\t\t<name>%s</name>\n" %(mesh.name.replace("<","-").replace(">","-"))
		if face.smooth: str += "\t\t<normal_smoothing>true</normal_smoothing>\n"
		else: str += "\t\t<normal_smoothing>false</normal_smoothing>\n"
		str += "\t\t<embedded>\n"
		str += "\t\t\t<expose_uv_set>\n"
		str += "\t\t\t\t<index>0</index>\n"
		str += "\t\t\t\t<name>uv</name>\n"
		str += "\t\t\t</expose_uv_set>\n"
		str += getVertices(vlist[:1400000])
	elif switch is 1: str = getVertices(vlist[1400000:2800000])
	elif switch is 2: str = getVertices(vlist[2800000:])
	else:
		str = getFaces()
		str += "\t\t</embedded>\n"
		str += "\t</mesh>\n\n"
	return str

#Wedge: Reworked with SSS/medium/epidermis/dermis/blend
def exportMaterial(mat):
	matName = mat.name
	if mat.mode & Material.Modes.RAYTRANSP:
		str = "\t<medium>\n"
		str += "\t\t<name>%s</name>\n" %matName
		str += "\t\t<precedence>%d</precedence>\n" %((1+mat.spec)*10)

		if matName.find(".epi") != -1:
			str += "\t\t<epidermis>\n"
			str += "\t\t\t<melanin_fraction>%.3f</melanin_fraction>\n" %mat.ref
			str += "\t\t\t<melanin_type_blend>%.3f</melanin_type_blend>\n" %mat.alpha
			str += "\t\t</epidermis>\n"
		elif matName.find(".der") != -1:
			str += "\t\t<dermis>\n"
			str += "\t\t\t<hemoglobin_fraction>%.3f</hemoglobin_fraction>\n" %mat.specTransp
			str += "\t\t</dermis>\n"
		else:
			str += "\t\t<basic>\n"
			str += "\t\t\t<ior>%.2f</ior>\n" %mat.IOR
			str += "\t\t\t<cauchy_b_coeff>%.3f</cauchy_b_coeff>\n" %mat.filter
			str += "\t\t\t<absorption_coefficient_spectrum>\n"
			if mat.ref > 0:
				str += "\t\t\t\t<uniform>\n"
				if mat.rayMirr > 0: str += "\t\t\t\t\t<value>%.0f</value>\n" %(mat.ref*1000)
				else: str += "\t\t\t\t\t<value>%.2f</value>\n" %(mat.ref*10)
				str += "\t\t\t\t</uniform>\n"
			else:
				str += "\t\t\t\t<rgb>\n"
				if mat.rayMirr > 0: str += "\t\t\t\t\t<rgb>%.0f %.0f %.0f</rgb>\n" %(abs(1-mat.R)*1000,abs(1-mat.G)*1000,abs(1-mat.B)*1000)
				else: str += "\t\t\t\t\t<rgb>%.3f %.3f %.3f</rgb>\n" %(abs(1-mat.R),abs(1-mat.G),abs(1-mat.B))
				str += "\t\t\t\t</rgb>\n"
			str += "\t\t\t</absorption_coefficient_spectrum>\n"

			#SSS on/off
			if mat.amb == 1:
				str += "\t\t\t<subsurface_scattering>\n"
				str += "\t\t\t\t<scattering_coefficient_spectrum>\n"
				if mat.specTransp > 0:
					str += "\t\t\t\t\t<uniform>\n"
					if mat.fresnelDepth: str += "\t\t\t\t\t\t<value>%.0f</value>\n" %(mat.specTransp*1000)
					else: str += "\t\t\t\t\t\t<value>%.2f</value>\n" %(mat.specTransp*10)
					str += "\t\t\t\t\t</uniform>\n"
				else:
					str += "\t\t\t\t\t<rgb>\n"
					if mat.fresnelDepth: str += "\t\t\t\t\t\t<rgb>%.0f %.0f %.0f</rgb>\n" %(abs(1-mat.specR)*1000,abs(1-mat.specG)*1000,abs(1-mat.specB)*1000)
					else: str += "\t\t\t\t\t\t<rgb>%.3f %.3f %.3f</rgb>\n" %(abs(1-mat.specR),abs(1-mat.specG),abs(1-mat.specB))
					str += "\t\t\t\t\t</rgb>\n"
				str += "\t\t\t\t</scattering_coefficient_spectrum>\n"
				str += "\t\t\t\t<phase_function>\n"
				if mat.fresnelTrans == 0: str += "\t\t\t\t\t<uniform/>\n"
				else:
					str += "\t\t\t\t\t<henyey_greenstein>\n"
					str += "\t\t\t\t\t\t<g_spectrum>\n"
					if mat.alpha < 1:
						str += "\t\t\t\t\t\t\t<uniform>\n"
						str += "\t\t\t\t\t\t\t\t<value>%.2f</value>\n" %(mat.alpha*10)
						str += "\t\t\t\t\t\t\t</uniform>\n"
					else:
						str += "\t\t\t\t\t\t\t<rgb>\n"
						str += "\t\t\t\t\t\t\t\t<rgb>%.3f %.3f %.3f</rgb>\n" %(mat.mirR,mat.mirG,mat.mirB)
						str += "\t\t\t\t\t\t\t</rgb>\n"
					str += "\t\t\t\t\t\t</g_spectrum>\n"
					str += "\t\t\t\t\t</henyey_greenstein>\n"
				str += "\t\t\t\t</phase_function>\n"
				str += "\t\t\t</subsurface_scattering>\n"
			str += "\t\t</basic>\n"
		str += "\t</medium>\n\n"

		#combine medium
		str += "\t<material>\n"
		str += "\t\t<name>%s</name>\n" %matName
		if mat.translucency == 0:
			str += "\t\t<glossy_transparent>\n"
			if mat.mode & Material.Modes.RAYMIRROR: str += "\t\t\t<exponent>1000000</exponent>\n"
			else: str += "\t\t\t<exponent>%d</exponent>\n" %(mat.hard*10)
			str += "\t\t\t<internal_medium_name>%s</internal_medium_name>\n" %matName
		else:
			str += "\t\t<specular>\n"
			str += "\t\t\t<transparent>true</transparent>\n"
			str += "\t\t\t<internal_medium_name>%s</internal_medium_name>\n" %matName
		for t in mat.getTextures():
			if (t != None) and (t.texco & Blender.Texture.TexCo['UV']) and (t.tex.type == Blender.Texture.Types.IMAGE):
				if t.mapto & Blender.Texture.MapTo['NOR']:
					if t.tex.image:
						str += "\t\t\t<bump_map>\n"
						str += "\t\t\t\t<uv_set>uv</uv_set>\n"
						str += "\t\t\t\t<path>%s</path>\n" %basename(t.tex.image.filename)
						str += "\t\t\t\t<b>%.4f</b>\n" %(t.norfac/10)
						str += "\t\t\t\t<exponent>1</exponent>\n"
						str += "\t\t\t</bump_map>\n"
		if mat.translucency == 0: str += "\t\t</glossy_transparent>\n"
		else: str += "\t\t</specular>\n"
	elif matName.find(".blend") != -1:
		blendA = matName.split('.', -2)[-2]
		blendB = matName.split('.', -1)[-1]
		#create material A
		str = "\t<material>\n"
		str += "\t\t<name>%s</name>\n" %blendA
		if mat.translucency < 1:
			str += "\t\t<diffuse>\n"
			str += "\t\t\t<colour>%.3f %.3f %.3f</colour>\n" %(mat.R,mat.G,mat.B)
			str += "\t\t</diffuse>\n"
		else: str += "\t\t<null_material/>\n"
		str += "\t</material>\n\n"

		#create material B
		str += "\t<material>\n"
		str += "\t\t<name>%s</name>\n" %blendB
		if mat.alpha == 1:
			str += "\t\t<diffuse>\n"
			str += "\t\t\t<colour>%.3f %.3f %.3f</colour>\n" %(mat.specR,mat.specG,mat.specB)
			str += "\t\t</diffuse>\n"
		else: str += "\t\t<null_material/>\n"
		str += "\t</material>\n\n"

		#create blend
		str += "\t<material>\n"
		str += "\t\t<name>%s</name>\n" %matName
		str += "\t\t<blend>\n"
		str += "\t\t\t<a_name>%s</a_name>\n" %blendA
		str += "\t\t\t<b_name>%s</b_name>\n" %blendB
		str += "\t\t\t<blend_factor>%.3f</blend_factor>\n" %mat.amb
		str += "\t\t\t<blend_map>\n"

		for t in mat.getTextures():
			if (t != None) and (t.texco & Blender.Texture.TexCo['UV']) and (t.tex.type == Blender.Texture.Types.IMAGE):
				if t.mapto & Blender.Texture.MapTo['COL']:
					if t.tex.image:
						str += "\t\t\t\t<uv_set>uv</uv_set>\n"
						str += "\t\t\t\t<path>%s</path>\n" %basename(t.tex.image.filename)
						str += "\t\t\t\t<exponent>%.2f</exponent>\n" %(t.colfac*10)
				elif t.mapto & Blender.Texture.MapTo['NOR']:
					if t.tex.image:
						str += "\t\t\t\t<uv_set>uv</uv_set>\n"
						str += "\t\t\t\t<path>%s</path>\n" %basename(t.tex.image.filename)
						str += "\t\t\t\t<b>%.4f</b>\n" %(t.norfac/10)
						str += "\t\t\t\t<exponent>1</exponent>\n"
		str += "\t\t\t</blend_map>\n"
		str += "\t\t</blend>\n"
	else:
		str = "\t<material>\n"
		str += "\t\t<name>%s</name>\n" %matName
		if mat.spec > 0.0001:
			str += "\t\t<phong>\n"
			if matName.find(".nk") != -1: str += "\t\t\t<nk_data>nkdata/%s</nk_data>\n" %(matName.split(".nk")[0]+".nk")
			else:
				str += "\t\t\t<diffuse>%.3f %.3f %.3f</diffuse>\n" %(mat.R,mat.G,mat.B)
				str += "\t\t\t<ior>%.2f</ior>\n" %mat.IOR
			if mat.mode & Material.Modes.RAYMIRROR: str += "\t\t\t<exponent>1000000</exponent>\n"
			else: str += "\t\t\t<exponent>%d</exponent>\n" %(mat.hard*10)
			str += "\t\t\t<fresnel_scale>%.3f</fresnel_scale>\n" %mat.ref
		else:
			str += "\t\t<diffuse>\n"
			str += "\t\t\t<colour>%.3f %.3f %.3f</colour>\n" %(mat.R,mat.G,mat.B)
		for t in mat.getTextures():
			if (t != None) and (t.texco & Blender.Texture.TexCo['UV']) and (t.tex.type == Blender.Texture.Types.IMAGE):
				if t.mapto & Blender.Texture.MapTo['COL']:
					if t.tex.image:
						str += "\t\t\t<albedo_texture>\n"
						str += "\t\t\t\t<uv_set>uv</uv_set>\n"
						str += "\t\t\t\t<path>%s</path>\n" %basename(t.tex.image.filename)
						str += "\t\t\t\t<exponent>%.2f</exponent>\n" %(t.colfac*10)
						str += "\t\t\t</albedo_texture>\n"
				elif t.mapto & Blender.Texture.MapTo['NOR']:
					if t.tex.image:
						str += "\t\t\t<bump_map>\n"
						str += "\t\t\t\t<uv_set>uv</uv_set>\n"
						str += "\t\t\t\t<path>%s</path>\n" %basename(t.tex.image.filename)
						str += "\t\t\t\t<b>%.4f</b>\n" %(t.norfac/10)
						str += "\t\t\t\t<exponent>1</exponent>\n"
						str += "\t\t\t</bump_map>\n"
		if mat.spec > 0: str += "\t\t</phong>\n"
		else: str += "\t\t</diffuse>\n"
	str += "\t</material>\n\n"
	return str

#exporting a object #zuegs: added support for Curves, Groups and other object-types #Wedge: Modified (only Mesh support)
def exportObject(obj, matrix, trigger):
	global meshlist
	if obj.type == "Mesh":
		mesh = Mesh.Get(obj.getData(1))
		if mesh.faces:
			str = ""
			meshname = mesh.name
			if meshname.find('.sphere') != -1 or meshname.find('.plane') != -1 or meshname.find('.rlight') != -1:
				if trigger is 0: str += "\t<!--%s-->\n" %(obj.name.replace("<","-").replace(">","-"))
			else:
				try: meshlist.index(meshname)
				except ValueError:
					if trigger is 0:
						str += "\t<!--%s-->\n" %(obj.name.replace("<","-").replace(">","-"))
						str += exportMesh(mesh,0)
					elif trigger is 1: str += exportMesh(mesh,1)
					elif trigger is 2: str += exportMesh(mesh,2)
					elif trigger is 3:
						str += exportMesh(mesh,3)
						meshlist.append(meshname)
			if trigger is 3:
				objectname = obj.name
				meshlight = 0
				if len(mesh.materials) > 0:
					mat0 = mesh.materials[0]
					if mat0.emit > 0: meshlight = 1
				if meshlight: print "Export: %s (Meshlight %s)" %(objectname, meshname)
				elif meshname.find('.sphere') != -1 or meshname.find('.plane') != -1 or meshname.find('.rlight') != -1: print "Export: %s (Primitive %s)" %(objectname, meshname)
				else: print "Export: %s (Mesh %s)" %(objectname, meshname)
#Wedge: added primitive support, I used part of Zuegs matrix fix here too (scale) (modified)
				scale  = abs((obj.SizeX + obj.SizeY + obj.SizeZ) / 3)
				if meshname.find('.sphere') != -1:
					str += "\t<sphere>\n"
					str += "\t\t<center>%.4f %.4f %.4f</center>\n" %obj.loc
					str += "\t\t<radius>%.4f</radius>\n" %scale
					if len(mesh.materials) > 0: str += "\t\t<material_name>%s</material_name>\n" %mat0.name
					else: str += "\t\t<material_name>Default</material_name>\n"
					str += "\t</sphere>\n\n"
				elif meshname.find('.plane') != -1:
					str += "\t<plane>\n"
					str += "\t\t<normal>0 0 1</normal>\n"
					str += "\t\t<dist>%s</dist>\n" %meshname.split('.', -1)[-1]
					if len(mesh.materials) > 0: str += "\t\t<material_name>%s</material_name>\n" %mat0.name
					else: str += "\t\t<material_name>Default</material_name>\n"
					str += "\t</plane>\n\n"
				elif meshname.find('.rlight') != -1:
					str += "\t<rectanglelight>\n"
					str += "\t\t<pos>%.4f %.4f %.4f</pos>\n" %obj.loc
					str += "\t\t<width>%s</width>\n" %meshname.split('.', -2)[-2]
					str += "\t\t<height>%s</height>\n" %meshname.split('.', -1)[-1]
					str += "\t\t<spectrum>\n"
					str += "\t\t\t<blackbody>\n"
					if meshlight: str += "\t\t\t\t<temperature>%d</temperature>\n" %(mat0.amb*10000)
					else: str += "\t\t\t\t<temperature>5000</temperature>\n"
					str += "\t\t\t\t<gain>1</gain>\n"
					str += "\t\t\t</blackbody>\n"
					str += "\t\t</spectrum>\n"
					str += "\t\t<efficacy_scale>\n"
					if meshlight:
						str += "\t\t\t<power_drawn>%d</power_drawn>\n" %mat0.hard
						str += "\t\t\t<overall_luminous_efficacy>%.1f</overall_luminous_efficacy>\n" %(mat0.spec*100)
					else:
						str += "\t\t\t<power_drawn>100</power_drawn>\n"
						str += "\t\t\t<overall_luminous_efficacy>17.5</overall_luminous_efficacy>\n"
					str += "\t\t</efficacy_scale>\n"
					str += "\t</rectanglelight>\n\n"
				else:
					if meshlight: str += "\t<meshlight>\n"
					else: str += "\t<model>\n"
					str += "\t\t<pos>%.4f %.4f %.4f</pos>\n" %obj.loc
#Wedge: Zuegs matrix fix added to code (modified)
					str += "\t\t<scale>%.4f</scale>\n" %scale
					str += "\t\t<rotation>\n"
					str += "\t\t\t<matrix>\n\t\t\t\t"
					for v in xrange(3):
						for z in xrange(3): str += "%f " %(matrix[z][v]/scale)
					str += "\n\t\t\t</matrix>\n"
					str += "\t\t</rotation>\n"
					str += "\t\t<mesh_name>%s</mesh_name>\n" %(meshname.replace("<","-").replace(">","-"))
					if meshlight:
						str += "\t\t<spectrum>\n"
						if (mat0.translucency > 0):
							str += "\t\t\t<peak>\n"
							str += "\t\t\t\t<peak_min>%d</peak_min>\n" %(mat0.translucency*1000)
							str += "\t\t\t\t<peak_width>%d</peak_width>\n" %(mat0.amb*1000)
							str += "\t\t\t\t<base_value>0</base_value>\n"
							str += "\t\t\t\t<peak_value>%.1f</peak_value>\n" %(mat0.emit*100)
							str += "\t\t\t</peak>\n"
						elif (mat0.translucency == 0 and mat0.amb > 0):
							str += "\t\t\t<blackbody>\n"
							str += "\t\t\t\t<temperature>%d</temperature>\n" %(mat0.amb*10000)
							str += "\t\t\t\t<gain>1</gain>\n"
							str += "\t\t\t</blackbody>\n"
						elif (mat0.translucency == 0 and mat0.amb == 0):
							str += "\t\t\t<rgb>\n"
							str += "\t\t\t\t<rgb>%.3f %.3f %.3f</rgb>\n" %(mat0.R, mat0.G, mat0.B)
							str += "\t\t\t</rgb>\n"
						str += "\t\t</spectrum>\n"
						str += "\t\t<efficacy_scale>\n"
						str += "\t\t\t<power_drawn>%d</power_drawn>\n" %mat0.hard
						str += "\t\t\t<overall_luminous_efficacy>%.1f</overall_luminous_efficacy>\n" %(mat0.spec*100)
						str += "\t\t</efficacy_scale>\n"
						str += "\t</meshlight>\n\n"
					else: str += "\t</model>\n\n"
			return str

#Assign event numbers to buttons and set default values
evtNoEvt = 0
evtExport = 1
evtExportAnim = 2
evtIsNumber = 3

MaxNumConsRej = Draw.Create(1000)
Bidir = Draw.Create(1)
MLT = Draw.Create(1)
Logging = Draw.Create(0)
IGI = Draw.Create(0)
SuperSF = Draw.Create(3)
DisplayPeriod = Draw.Create(30)
Polarisation = Draw.Create(0)
PolarAngle = Draw.Create(0)
HaltTime = Draw.Create(-1)
ImageSavePeriod = Draw.Create(30)
NumThreads = Draw.Create(1)
SaveUTMExr = Draw.Create(0)
SaveTMExr = Draw.Create(0)
MatFile = Draw.Create(0)

ApertureRadius = Draw.Create(.002)
WhiteBalance = Draw.Create(5)
WhiteBalanceV = {}
WhiteBalanceV[0] = "A"
WhiteBalanceV[1] = "F11"
WhiteBalanceV[2] = "F2"
WhiteBalanceV[3] = "B"
WhiteBalanceV[4] = "D50"
WhiteBalanceV[5] = "E"
WhiteBalanceV[6] = "D55"
WhiteBalanceV[7] = "F7"
WhiteBalanceV[8] = "D65"
WhiteBalanceV[9] = "C"
WhiteBalanceV[10] = "D75"
WhiteBalanceV[11] = "9300"
AF = Draw.Create(1)
BloomWeight = Draw.Create(.1)
BloomRadius = Draw.Create(.05)
EnvType = Draw.Create(1)
Turbidity = Draw.Create(2.0)
GroundPlane = Draw.Create(0)

EnvFile = Draw.Create("")
EnvMapType = Draw.Create(0)
EnvMapTypeV = {}
EnvMapTypeV[0] = "spherical"
EnvMapTypeV[1] = "latlong"
EnvGain = Draw.Create(1.0)
EnvWidth = Draw.Create(640)

ToneMapType = Draw.Create(1)
ToneMapScale = Draw.Create(".005")
ToneMapPreScale = Draw.Create("8.0")
ToneMapPostScale = Draw.Create("1.0")
ToneMapBurn = Draw.Create(15.0)

def update_Registry():
	d = {}
	d['apertureradius'] = ApertureRadius.val
	d['whitebalance'] = WhiteBalance.val
	d['bloomweight'] = BloomWeight.val
	d['bloomradius'] = BloomRadius.val
	d['af'] = AF.val
	d['maxnumconsrej'] = MaxNumConsRej.val
	d['bidir'] = Bidir.val
	d['mlt'] = MLT.val
	d['logging'] = Logging.val
	d['igi'] = IGI.val
	d['supersf'] = SuperSF.val
	d['displayperiod'] = DisplayPeriod.val
	d['polarisation'] = Polarisation.val
	d['polarangle'] = PolarAngle.val
	d['halttime'] = HaltTime.val
	d['imagesaveperiod'] = ImageSavePeriod.val
	d['numthreads'] = NumThreads.val
	d['groundplane'] = GroundPlane.val
	d['saveutmexr'] = SaveUTMExr.val
	d['savetmexr'] = SaveTMExr.val
	d['tonemaptype'] = ToneMapType.val
	d['tonemapscale'] = ToneMapScale.val
	d['tonemapprescale'] = ToneMapPreScale.val
	d['tonemappostscale'] = ToneMapPostScale.val
	d['tonemapburn'] = ToneMapBurn.val
	d['envtype'] = EnvType.val
	d['envfile'] = EnvFile.val
	d['envmaptype'] = EnvMapType.val
	d['envgain'] = EnvGain.val
	d['envwidth'] = EnvWidth.val
	d['turbidity'] = Turbidity.val
	d['matfile'] = MatFile.val
	Blender.Registry.SetKey('BlenderIndigo', d, True)

rdict = Blender.Registry.GetKey('BlenderIndigo', True)
if rdict:
	try:
		ApertureRadius.val = rdict['apertureradius']
		WhiteBalance.val = rdict['whitebalance']
		BloomWeight.val = rdict['bloomweight']
		BloomRadius.val = rdict['bloomradius']
		AF.val = rdict['af']
		MaxNumConsRej.val = rdict['maxnumconsrej']
		Bidir.val = rdict['bidir']
		MLT.val = rdict['mlt']
		Logging.val = rdict['logging']
		IGI.val = rdict['igi']
		SuperSF.val = rdict['supersf']
		DisplayPeriod.val = rdict['displayperiod']
		Polarisation.val = rdict['polarisation']
		PolarAngle.val = rdict['polarangle']
		HaltTime.val = rdict['halttime']
		ImageSavePeriod.val = rdict['imagesaveperiod']
		NumThreads.val = rdict['numthreads']
		GroundPlane.val = rdict['groundplane']
		SaveUTMExr.val = rdict['saveutmexr']
		SaveTMExr.val = rdict['savetmexr']
		ToneMapType.val = rdict['tonemaptype']
		ToneMapScale.val = rdict['tonemapscale']
		ToneMapPreScale.val = rdict['tonemapprescale']
		ToneMapPostScale.val = rdict['tonemappostscale']
		ToneMapBurn.val = rdict['tonemapburn']
		EnvType.val = rdict['envtype']
		EnvFile.val = rdict['envfile']
		EnvMapType.val = rdict['envmaptype']
		EnvGain.val = rdict['envgain']
		EnvWidth.val = rdict['envwidth']
		Turbidity.val = rdict['turbidity']
		MatFile.val = rdict['matfile']
	except: update_Registry()

def gui():
	global Bidir, MLT, Logging, IGI, SuperSF, DisplayPeriod, Polarisation, PolarAngle, ApertureRadius, GroundPlane, MatFile, SaveUTMExr, SaveTMExr, ToneMapType, ToneMapScale, ToneMapPreScale
	global ToneMapPostScale, ToneMapBurn, MaxNumConsRej, EnvType, EnvFile, EnvMapType, EnvGain, EnvWidth, Turbidity, WhiteBalance, BloomWeight, BloomRadius, AF, HaltTime, ImageSavePeriod, NumThreads

	BGL.glColor3f(0,0,0)
	BGL.glRasterPos2i(100,270)
	Draw.Text("Indigo 0.7-Final Export 1.21	(Quit: ESC)", 'large')
	BGL.glRasterPos2i(60,255)
	Draw.Text("Camera:")
	Draw.BeginAlign()
	AF = Draw.Toggle("Autofocus", evtNoEvt, 10, 230, 77, 18, AF.val, "If pressed, camera uses autofocus.")
	WhiteBalance = Draw.Menu("White Balance: %t|A %x0|F11 %x1|F2 %x2|B %x3|D50 %x4|E %x5|D55 %x6|F7 %x7|D65 %x8|C %x9|D75 %x10|9300 %x11", evtNoEvt, 89, 230, 76, 18, WhiteBalance.val, "From top down, sorted by temperature: blue fading to red.")
	ApertureRadius = Draw.Number("Aperture Radius:", evtNoEvt, 10, 210, 155, 18, ApertureRadius.val, 0.0, 1.0, "In meters, 0 is off. Defines radius of camera lens, or radius of camera aperture. Larger radius means more depth of field.")
	BloomWeight = Draw.Number("Bloom Weight:", evtNoEvt, 10, 190, 155, 18, BloomWeight.val, 0.0, 9.999, "Glowing effect, 0 = off")
	BloomRadius = Draw.Number("Bloom Radius:", evtNoEvt, 10, 170, 155, 18, BloomRadius.val, 0.0, 9.999, "Glowing effect, 0 = off")
	Polarisation = Draw.Toggle("Polarisation", evtNoEvt, 10, 150, 155, 18, Polarisation.val, "Filter light wave direction")
	if Polarisation.val: PolarAngle = Draw.Number("Polarisation Angle:", evtNoEvt, 10, 130, 155, 18, PolarAngle.val, 0, 359, "Angle of Polarisation")
	Draw.EndAlign()

	BGL.glRasterPos2i(265,255)
	Draw.Text("Render:")
	Draw.BeginAlign()
	Bidir = Draw.Toggle("Bidir", evtNoEvt, 175, 230, 110, 18, Bidir.val, "If pressed, use bidirectional path tracing, if not, use only path tracing.")
	MLT = Draw.Toggle("MLT", evtNoEvt, 287, 230, 110, 18, MLT.val, "If pressed, use MLT, if not, use only path tracing.")
	Logging = Draw.Toggle("Log", evtNoEvt, 175, 210, 110, 18, Logging.val, "Write to log.txt if pressed")
	SaveUTMExr = Draw.Toggle("Save UTM EXR", evtNoEvt, 287, 210, 110, 18, SaveUTMExr.val, "Save untonemapped EXR file")
	IGI = Draw.Toggle("IGI", evtNoEvt, 175, 190, 110, 18, IGI.val, "Save IGI file")
	SaveTMExr = Draw.Toggle("Save TM EXR", evtNoEvt, 287, 190, 110, 18, SaveTMExr.val, "Save tonemapped EXR file")
	ImageSavePeriod = Draw.Number("Save:", evtNoEvt, 175, 170, 110, 18, ImageSavePeriod.val, 30, 900, "Save Period in seconds")
	DisplayPeriod = Draw.Number("Display:", evtNoEvt, 287, 170, 110, 18, DisplayPeriod.val, 30, 900, "Display Period in seconds")
	HaltTime = Draw.Number("Halt Time:", evtNoEvt, 175, 150, 127, 18, HaltTime.val, -1, 604800, "Halt Time in seconds (-1 = off)")
	NumThreads = Draw.Number("Threads:", evtNoEvt, 304, 150, 93, 18, NumThreads.val, 1, 32, "Number of Threads")
	MaxNumConsRej = Draw.Number("Max Consec Rejections:", evtNoEvt, 175, 130, 222, 18, MaxNumConsRej.val, 100, 10000, "Lower values mean more biased calculations")
	SuperSF = Draw.Number("Super Sample Factor:", evtNoEvt, 175, 110, 222, 18, SuperSF.val, 0, 8, "Reduces aliasing (Higher value = more RAM)")
	Draw.EndAlign()

	BGL.glRasterPos2i(442,255)
	Draw.Text("Options:")
	Draw.BeginAlign()
	GroundPlane = Draw.Toggle("Ground Plane", evtNoEvt, 407, 230, 114, 18, GroundPlane.val, "Place infinite large ground plane at 0,0,0")
	MatFile = Draw.Toggle("Separate Materials", evtNoEvt, 407, 210, 114, 18, MatFile.val, "Save all material settings to a separate file with a \"-materials\" extension")
	Draw.EndAlign()

	BGL.glRasterPos2i(55,95)
	Draw.Text("Tonemap:")
	Draw.BeginAlign()
	ToneMapType = Draw.Menu("Tonemap Type: %t|Linear %x0|Reinhard %x1", evtNoEvt, 10, 70, 155, 18, ToneMapType.val, "Type of tonemapping")
	if ToneMapType.val is 0: ToneMapScale = Draw.String("Scale: ", evtIsNumber, 10, 50, 155, 18, ToneMapScale.val, 9, "Scale pixel value")
	else:
		ToneMapPreScale = Draw.String("PreScale: ", evtIsNumber, 10, 50, 155, 18, ToneMapPreScale.val, 9, "Middle-grey luminance scaling")
		ToneMapPostScale = Draw.String("PostScale: ", evtIsNumber, 10, 30, 155, 18, ToneMapPostScale.val, 9, "Lesser luminance pixels mapped to white")
		ToneMapBurn = Draw.Number("Burn:", evtNoEvt, 10, 10, 155, 18, ToneMapBurn.val, 1.0, 15, "Determines luminance where clipping occurs, small value = more burn")
	Draw.EndAlign()

	BGL.glRasterPos2i(250,95)
	Draw.Text("Environment:")
	Draw.BeginAlign()
	EnvType = Draw.Menu("Environment Type: %t|Background %x0|SunSky %x1|Map %x2", evtNoEvt, 175, 70, 222, 18, EnvType.val, "Enviroment type")
	if EnvType.val is 1: Turbidity = Draw.Number("Sky Turbidity:", evtNoEvt, 175, 50, 222, 18, Turbidity.val, 1.7, 6.7, "Sky Turbidity")
	elif EnvType.val is 2:
		EnvFile = Draw.String("Probe: ", evtNoEvt, 175, 50, 222, 18, EnvFile.val, 50, "File name of the raw/exr probe")
		EnvGain = Draw.Number("Gain:", evtNoEvt, 175, 30, 111, 18, EnvGain.val, 0.0, 100.0, "Gain")
		EnvMapType = Draw.Menu("Map Type: %t|Spherical %x0|LatLong %x1", evtNoEvt, 288, 30, 109, 18, EnvMapType.val, "Map type of the probe")
		if EnvMapType.val is 0: EnvWidth = Draw.Number("Width:", evtNoEvt, 175, 10, 222, 18, EnvWidth.val, 1, 10000, "Width")
	Draw.EndAlign()

	BGL.glRasterPos2i(445,95)
	Draw.Text("Export:")
	Draw.BeginAlign()
	Draw.PushButton("Export Anim", evtExportAnim, 407, 70, 114, 18, "Open file dialog and export animation (careful: takes a lot of diskspace!)")
	Draw.PushButton("Export", evtExport, 407, 50, 114, 18, "Open file dialog and export")
	Draw.EndAlign()

def event(evt,val):
	if evt == Draw.ESCKEY: Draw.Exit()

def buttonEvt(evt):
	def save_indigo(filename, unindexedname):
		def collectObjectMaterials(obj):
			if obj.type == "Mesh":
				materials = obj.data.getMaterials()
				meshlight = 0
				if len(materials) > 0:
					if materials[0].emit > 0: meshlight = 1
				if meshlight == 0:
					for mat in materials:
						if mat.name not in matnames: matnames.append(mat.name)
		global meshlist
		time1 = Blender.sys.time()
		print "\nIndigo Export - Saving to " + filename
		file = open(filename, 'w')
		file.write("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n")
		file.write("<scene>\n")

		myScene = Scene.GetCurrent()
		SizeX = myScene.getRenderingContext().sizeX
		SizeY = myScene.getRenderingContext().sizeY
		renderSize = myScene.getRenderingContext().renderwinSize
		file.write("\t<renderer_settings>\n")
		file.write("\t\t<width>%d</width>\n" %(SizeX * renderSize / 100))
		file.write("\t\t<height>%d</height>\n" %(SizeY * renderSize / 100))
		if MLT.val: file.write("\t\t<metropolis>true</metropolis>\n")
		else: file.write("\t\t<metropolis>false</metropolis>\n")
		file.write("\t\t<max_num_consec_rejections>%d</max_num_consec_rejections>\n" %MaxNumConsRej.val)
		if Bidir.val: file.write("\t\t<bidirectional>true</bidirectional>\n")
		else: file.write("\t\t<bidirectional>false</bidirectional>\n")
		file.write("\t\t<halt_time>%d</halt_time>\n" %HaltTime.val)
		if Logging.val: file.write("\t\t<logging>true</logging>\n")
		else: file.write("\t\t<logging>false</logging>\n")
		file.write("\t\t<display_period>%d</display_period>\n" %DisplayPeriod.val)
		file.write("\t\t<image_save_period>%d</image_save_period>\n" %ImageSavePeriod.val)
		if SaveTMExr.val: file.write("\t\t<save_tonemapped_exr>true</save_tonemapped_exr>\n")
		else: file.write("\t\t<save_tonemapped_exr>false</save_tonemapped_exr>\n")
		if SaveUTMExr.val: file.write("\t\t<save_untonemapped_exr>true</save_untonemapped_exr>\n")
		else: file.write("\t\t<save_untonemapped_exr>false</save_untonemapped_exr>\n")
		if IGI.val: file.write("\t\t<save_igi>true</save_igi>\n")
		else: file.write("\t\t<save_igi>false</save_igi>\n")
		file.write("\t\t<num_threads>%d</num_threads>\n" %NumThreads.val)
		if Polarisation.val: file.write("\t\t<polarisation>true</polarisation>\n")
		else: file.write("\t\t<polarisation>false</polarisation>\n")
		file.write("\t\t<super_sample_factor>%d</super_sample_factor>\n" %SuperSF.val)
		file.write("\t</renderer_settings>\n\n")

		file.write("\t<tonemapping>\n")
		if ToneMapType.val is 1:
			file.write("\t\t<reinhard>\n")
			file.write("\t\t\t<pre_scale>%f</pre_scale>\n" %float(ToneMapPreScale.val))
			file.write("\t\t\t<post_scale>%f</post_scale>\n" %float(ToneMapPostScale.val))
			file.write("\t\t\t<burn>%.2f</burn>\n" %ToneMapBurn.val)
			file.write("\t\t</reinhard>\n")
		else:
			file.write("\t\t<linear>\n")
			file.write("\t\t\t<scale>%f</scale>\n" %float(ToneMapScale.val))
			file.write("\t\t</linear>\n")
		file.write("\t</tonemapping>\n\n")

		if EnvType.val is 0:
			worldcolor = Blender.World.Get('World').hor
			file.write("\t<background>\n")
			file.write("\t\t<spectrum>\n")
			file.write("\t\t\t<rgb>\n")
			file.write("\t\t\t\t<rgb>%.3f %.3f %.3f</rgb>\n" %(worldcolor[0], worldcolor[1], worldcolor[2]))
			file.write("\t\t\t</rgb>\n")
			file.write("\t\t</spectrum>\n")
			file.write("\t</background>\n\n")

		#get all objects in selected layers
		activelayers = Window.ViewLayers()
		sunObj = ""
		camObj = ""
		object_list = []
		matnames = []
		for obj in myScene.objects:
			loopOnce = 0
			for layer in obj.layers:
				if loopOnce is 0:
					loopOnce = 1
					if layer in activelayers:
						if obj.type == 'Mesh':
							object_list.append(obj)
							collectObjectMaterials(obj)
						elif obj.type == 'Lamp': sunObj = obj
						elif obj.type == 'Camera': camObj = obj

		dummyMat = "\t<material>\n\t\t<name>Default</name>\n\t\t<diffuse>\n\t\t\t<colour>1 1 1</colour>\n\t\t</diffuse>\n\t</material>\n\n"
		#Materialfile include
		if MatFile.val is 0:
			#Define a dummy material and Process Materials
			file.write(dummyMat)
			materials = Material.Get()
			for mat in materials:
				if mat.name in matnames: file.write(exportMaterial(mat))
		else:
			materialfile = makename(unindexedname, '-materials.igs')
			materialinclude = basename(materialfile)
			file.write("\t<include>\n")
			file.write("\t\t<pathname>%s</pathname>\n" %materialinclude)
			file.write("\t</include>\n\n")

		if EnvType.val is 1:
			if sunObj != '':
				print "Export: Sun"
				invmatrix = sunObj.getInverseMatrix()
				file.write("\t<skylight>\n")
				file.write("\t\t<sundir>%.4f %.4f %.4f</sundir>\n" %(invmatrix[0][2], invmatrix[1][2], invmatrix[2][2]))
				file.write("\t\t<turbidity>%.3f</turbidity>\n" %Turbidity.val)
				file.write("\t</skylight>\n\n")
		elif EnvType.val is 2:
			file.write("\t<env_map>\n")
			file.write("\t\t<%s>\n" %EnvMapTypeV[EnvMapType.val])
			file.write("\t\t\t<path>%s</path>\n" %EnvFile.val)
			file.write("\t\t\t<gain>%.1f</gain>\n" %EnvGain.val)
			if EnvMapType.val is 0: file.write("\t\t\t<width>%d</width>\n" %EnvWidth.val)
			file.write("\t\t</%s>\n" %EnvMapTypeV[EnvMapType.val])
			file.write("\t</env_map>\n\n")

		if camObj != '':
			print "Export: Camera"
			camData = Camera.Get(camObj.getData(1))
			matrix = camObj.matrix
			pos = matrix[3]
			forwards = matrix[2]
			up = matrix[1]
			file.write("\t<camera>\n")
			file.write("\t\t<pos>%.4f %.4f %.4f</pos>\n" %(pos[0], pos[1], pos[2]))
			file.write("\t\t<up>%.4f %.4f %.4f</up>\n" %(up[0], up[1], up[2]))
			file.write("\t\t<forwards>%.4f %.4f %.4f</forwards>\n" %(-forwards[0], -forwards[1], -forwards[2]))
			cameraLens = camData.lens
			aspect = float(SizeX) / float(SizeY)
			film = .001 * cameraLens
#zuegs: correct aspect stuff for portrait-mode #Wedge: (modified)
			if aspect < 1.0: lensfocal = (cameraLens * film / aspect)/32
			else: lensfocal = (cameraLens * film)/32
			file.write("\t\t<aperture_radius>%.3f</aperture_radius>\n" %ApertureRadius.val)
			file.write("\t\t<focus_distance>%.2f</focus_distance>\n" %camData.dofDist)
			file.write("\t\t<aspect_ratio>%.2f</aspect_ratio>\n" %aspect)
			file.write("\t\t<sensor_width>%.3f</sensor_width>\n" %film)
			file.write("\t\t<lens_sensor_dist>%.4f</lens_sensor_dist>\n" %lensfocal)
			file.write("\t\t<white_balance>%s</white_balance>\n" %WhiteBalanceV[WhiteBalance.val])
			if AF.val: file.write("\t\t<autofocus/>\n")
			if Polarisation.val:
				file.write("\t\t<polarising_filter>\n")
				file.write("\t\t\t<angle>%d</angle>\n" %PolarAngle.val)
				file.write("\t\t</polarising_filter>\n")
			file.write("\t\t<bloom>\n")
			file.write("\t\t\t<weight>%.3f</weight>\n" %BloomWeight.val)
			file.write("\t\t\t<radius>%.3f</radius>\n" %BloomRadius.val)
			file.write("\t\t</bloom>\n")
			file.write("\t</camera>\n\n")

		if GroundPlane.val is 1:
			file.write("\t<plane>\n")
			file.write("\t\t<normal>0 0 1</normal>\n")
			file.write("\t\t<dist>0</dist>\n")
			file.write("\t\t<material_name>Default</material_name>\n")
			file.write("\t</plane>\n\n")

		#Process Meshes
		meshlist = []
		for obj in object_list:
			file.write(exportObject(obj,obj.matrix,0))
			file.write(exportObject(obj,obj.matrix,1))
			file.write(exportObject(obj,obj.matrix,2))
			file.write(exportObject(obj,obj.matrix,3))

		file.write("</scene>")
		file.close()

		if MatFile.val and not MatSaved:
			print "Saving Material File"
			file = open(materialfile, 'w')
			file.write("<scenedata>\n")

			#Define a dummy material and Process Materials
			file.write(dummyMat)
			materials = Material.Get()
			for mat in materials:
				if mat.name in matnames: file.write(exportMaterial(mat))
			file.write("</scenedata>")
			file.close()
		print "Done - Export time: %.2f" %(Blender.sys.time()-time1)

	def saveChoice(filename):
		global MatSaved
		MatSaved = 0
		if evt is evtExportAnim:
			for i in xrange(Blender.Get('staframe'), Blender.Get('endframe')+1):
				Blender.Set('curframe', i)
				Blender.Redraw()
				frameindex = "-" + str(i) + ".igs"
				indexedname = makename(filename, frameindex)
				unindexedname = filename
				save_indigo(indexedname, unindexedname)
				MatSaved = 1
		elif evt is evtExport:
			unindexedname = filename
			save_indigo(filename, unindexedname)

	if evt is evtExport: Blender.Window.FileSelector(saveChoice, "Export", Blender.Get('filename')[: -len(Blender.Get('filename').split('.', -1)[-1])] + "igs")
	elif evt is evtExportAnim: Blender.Window.FileSelector(saveChoice, "Export Animation", Blender.Get('filename')[: -len(Blender.Get('filename').split('.', -1)[-1])] + "igs")
	elif evt is evtNoEvt:
		Draw.Redraw()
		update_Registry()
	elif evt is evtIsNumber:
		global ToneMapScale, ToneMapPreScale, ToneMapPostScale
		try: float(ToneMapScale.val)
		except: ToneMapScale.val = ".005"
		try: float(ToneMapPreScale.val)
		except: ToneMapPreScale.val = "8.0"
		try: float(ToneMapPostScale.val)
		except: ToneMapPostScale.val = "1.0"
		Draw.Redraw()
		update_Registry()

Draw.Register(gui,event,buttonEvt)